#!/bin/bash
model_dir="models/"
embeddings_dir="word-embeddings/"
treebanks="treebanks/"
logfile="../../l2r-baseline-dev-times-cpu-time.csv"
echo "model,total time 1,parse time 1,total time 2,parse time 2,total time 3,parse time 3,total time 4,parse time 4,total time 5,parse time 5" > $logfile
for treebank_dir in $treebanks/UD_*
do
    treebank=$(basename $treebank_dir)
    embeddings=$(echo "$embeddings_dir$treebank/vectors")
    dev=$(ls $treebank_dir/*test.conllu)
    out_dev=$(basename $dev)
    
    for n_layers in 1 2 
    do
	for n_nodes in 200 300 400 500 
	do
	    
	    let n=$n_layers+1
	    let m=$n_nodes+$n_nodes
	    pred=$(echo $n-$m-$out_dev)
	    model_path=$(echo "$model_dir$treebank-$n-$m")
	    echo $model_path
	    times=""
	    for x in 1 2 3 4 5
	    do
		echo $x
		times_new=$(CUDA_VISIBLE_DEVICES="-1"  taskset --cpu-list $x python -u parsing.py --mode parse\
				    --config configs/parsing/l2r.json --num_epochs 200 \
				    --batch_size 128 \
				    --opt adam --learning_rate 0.001 --lr_decay 0.999997\
				    --beta1 0.9 --beta2 0.9 --eps 1e-4 --grad_clip 5.0 \
				    --loss_type token --warmup_steps 40 --reset 20 \
				    --weight_decay 0.0 --unk_replace 0.5 --beam 10 \
				    --word_path $embeddings \
				    --char_embedding random \
				    --punctuation '.' '``' "''" ':' ',' \
				    --patience 10 \
				    --test $dev \
				    --fgold "gold/$pred" \
				    --fpred "pred/$pred" \
				    --model_path $model_path \
				    --hidden_size $n_nodes \
				    --remove_cycles \
				    --encoder_layers $n_layers | grep "TIMES" | awk '{ print $2 }')
		times="${times},${times_new}"
	    done
	    echo $treebank-$n-$m$times >> $logfile
	done
    done
done
