#!/bin/bash
model_dir="models/"
embeddings_dir="word-embeddings/"
treebanks="treebanks/"
logfile="../../training.csv"
energy_logfile="../../energy.csv"

for treebank_dir in $treebanks/UD_*
do
    treebank=$(basename $treebank_dir)
    embeddings=$(echo "$embeddings_dir$treebank/vectors")
    train=$(ls $treebank_dir/*train.conllu)
    dev=$(ls $treebank_dir/*dev.conllu)
    for n_layers in 1 2 
    do
	for n_nodes in 200 300 400 500 
	do
	    let n=$n_layers+1
	    let m=$n_nodes+$n_nodes
	    model_path=$(echo "$model_dir$treebank-$n-$m")
	    echo $model_path
	    CUDA_VISIBLE_DEVICES=1 python -u parsing.py --mode train\
				--config configs/parsing/l2r.json --num_epochs 200 \
				--batch_size 32 \
				--opt adam --learning_rate 0.001 --lr_decay 0.999997\
				--beta1 0.9 --beta2 0.9 --eps 1e-4 --grad_clip 5.0 \
				--loss_type token --warmup_steps 40 --reset 20 \
				--weight_decay 0.0 --unk_replace 0.5 --beam 10 \
				--word_path $embeddings \
				--char_embedding random \
				--punctuation '.' '``' "''" ':' ',' \
				--patience 10 \
				--log_energy $energy_logfile \
				--log_path $logfile \
				--train $train \
				--dev $dev \
				--model_path $model_path \
				--hidden_size $n_nodes \
	                        --encoder_layers $n_layers
	done
    done
done
