# -*- coding: utf-8 -*-

from parser import BiaffineParser, Model
from parser.utils import Corpus
from parser.utils import reintroduce
from parser.utils.data import TextDataset, batchify
import time
import torch
import tempfile

class Forge(object):

    def add_subparser(self, name, parser):
        subparser = parser.add_parser(
            name, help='Use a trained model to make predictions.'
        )
        subparser.add_argument('--batch-size', default=5000, type=int,
                               help='batch size')
        subparser.add_argument('--fdata', default='data/ptb/test.conllx',
                               help='path to dataset')
        subparser.add_argument('--fpred', default='pred.conllx',
                               help='path to predicted result')
        subparser.add_argument('--fgoldtrain', default=False,
                               help='path to predicted result')
        subparser.add_argument('--number_to_forge', default=False,
                               help='path to predicted result')
        subparser.add_argument('--withhold', action='store_true', default=False, help='Whether to withhold 10% of gold data or not.')
        return subparser

    def __call__(self, config):
        print("Load the model")
        vocab = torch.load(config.vocab)
        parser = BiaffineParser.load(config.model)
        model = Model(vocab, parser)

        print("Load the dataset")
        corpus = Corpus.load(config.fdata)
        ### check it works for a reasonable number of trees
        #corpus.sentences = corpus.sentences[:10000]
        if config.fgoldtrain:
            goldtrain = Corpus.load(config.fgoldtrain)
            if config.withhold:
                numberToWithold = int(len(goldtrain.sentences)/50)
                if numberToWithold < 5:
                    numberToWithold = 5
                print("Withholding " + str(numberToWithold) + " gold training sentences.")
                goldtrain.sentences = goldtrain.sentences[:-numberToWithold]
                number_of_silver_trees_to_forge = len(goldtrain)
        elif config.number_to_forge:
            number_of_silver_trees_to_forge = int(config.number_to_forge)
        else:
            number_of_silver_trees_to_forge = 100
        print("Creating " + str(number_of_silver_trees_to_forge) + " new trees.")
        dataset = TextDataset(vocab.numericalize(corpus, False))
        # set the data loader
        start_time = time.time()
        loader = batchify(dataset, config.batch_size)

        print("Make predictions on the dataset")
        corpus.heads, corpus.rels, entropies = model.forge(loader)
        corpus.set_entropies(entropies)
        timeTotal = time.time() - start_time
        speed = float(len(corpus.heads)) /timeTotal
        print("Speed: " + str(speed) + " sent/s")
        print(f"Save the predicted result to {config.fpred}")
        #tmp = tempfile.NamedTemporaryFile(delete=False)
        corpus.save_forged_data(config.fpred, number_of_silver_trees_to_forge)
        
