# -*- coding: utf-8 -*-

from parser import BiaffineParser, Model
from parser.utils import Corpus, RawCorpus
from parser.utils import reintroduce
from parser.utils.data import TextDataset, batchify
from parser.utils.seq_lab import seq_lab_decoder_corpus
import time
import torch
import tempfile

class Predict(object):
    
    def add_subparser(self, name, parser):
        subparser = parser.add_parser(
            name, help='Use a trained model to make predictions.'
        )
        subparser.add_argument('--batch-size', default=128, type=int,
                               help='batch size')
        subparser.add_argument('--fdata', default='data/ptb/test.conllx',
                               help='path to dataset')
        subparser.add_argument('--fpred', default='pred.conllx',
                               help='path to predicted result')
        
        
        return subparser

    def __call__(self, config):
        time_t0 = time.time()
        print("Load the model")
        vocab = torch.load(config.vocab)
        parser = BiaffineParser.load(config.model)
        char_lemma = parser.config.char_lemma
        print("char_lemma",char_lemma)
        model = Model(vocab, parser)
        print("Load the dataset")
        corpus = Corpus.load(config.fdata, raw=True)
        dataset = TextDataset(vocab.numericalize(corpus, training=False, char_lemma=char_lemma))
        # set the data loader
        start_time = time.time()
        loader = batchify(dataset, config.batch_size)

        print("Make predictions on the dataset")
        start_parse_time = time.time()
        pred_encs, pred_rels = model.predict(loader)
        corpus.heads, corpus.rels = seq_lab_decoder_corpus(pred_encs, pred_rels)
        parse_time = time.time() - start_parse_time   
        print(f"Save the predicted result to {config.fpred}")
        #tmp = tempfile.NamedTemporaryFile(delete=False)
        #corpus.save(tmp.name)
        ### for reintroducing 1-2 and 1.5 entries for evaluation
        #reintroduce(config.fdata, tmp.name, config.fpred)
        corpus.save(config.fpred)
        time_total = time.time() - time_t0
        print("TIMES", str(time_total) + "," + str(parse_time))
        #timeTotal = time.time() - start_time
        #speed = float(len(corpus.heads)) /timeTotal
        #print("Speed: " + str(speed) + " sent/s")
