# -*- coding: utf-8 -*-

from parser import BiaffineParser, Model
from parser.utils import Corpus
from parser.utils import reintroduce
from parser.utils.data import TextDataset, batchify
import time
import torch
import tempfile

class Score(object):

    def add_subparser(self, name, parser):
        subparser = parser.add_parser(
            name, help='Use a trained model to make predictions.'
        )
        subparser.add_argument('--batch-size', default=100, type=int,
                               help='batch size')
        subparser.add_argument('--fdata', default='data/ptb/test.conllx',
                               help='path to dataset')
        subparser.add_argument('--fscores', default='scores.txt',
                               help='path to predicted result')

        return subparser

    def __call__(self, config):
        print("Load the model")
        vocab = torch.load(config.vocab)
        parser = BiaffineParser.load(config.model)
        model = Model(vocab, parser)

        print("Load the dataset")
        corpus = Corpus.load(config.fdata)
        dataset = TextDataset(vocab.numericalize(corpus))#, False))
        # set the data loader
        start_time = time.time()
        loader = batchify(dataset, config.batch_size)
        
        print("Make predictions on the dataset")
        entropies = model.score(loader)
        timeTotal = time.time() - start_time
        speed = float(len(corpus.heads)) /timeTotal
        print("Speed: " + str(speed) + " sent/s")
        print(f"Save the scores to {config.fscores}")
        corpus.save_entropy_scores(config.fscores, entropies)
        
        ### for reintroducing 1-2 and 1.5 entries for evaluation
        #reintroduce(config.fdata, tmp.name, config.fpred)
