# -*- coding: utf-8 -*-

from parser.metric import Metric
from parser.utils import CEOR
import numpy as np
import torch
import torch.nn as nn
from torch import log, exp
import torch.nn.functional as F
from scipy.stats import entropy as calc_entropy
from math import log


class LMModel(object):

    def __init__(self, vocab, lm):
        super(LMModel, self).__init__()

        self.vocab = vocab
        self.lm = lm
        self.criterion = nn.CrossEntropyLoss()

    def train(self, loader):
        self.lm.train()

        for words, _, chars, _, _ in loader:
            self.optimizer.zero_grad()

            mask_f = words.ne(self.vocab.pad_index)
            mask_b = words.ne(self.vocab.pad_index)
            # ignore the last token of each sentence in forward dir
            mask_f[:, -1] = 0
            # ignore the first token of each sentence in backward dir
            mask_b[:, 0] = 0

            f_logits, b_logits = self.lm(words, chars)

            f_logits, b_logits = f_logits[mask_f], b_logits[mask_b]

            # offset the targets
            pad = torch.zeros_like(words[:, 0:1])
            gold_f = torch.cat([words[:, 1:], pad], dim=1)
            gold_b = torch.cat([pad, words[:, :-1]], dim=1)
            # the first and last token in the bwd and fwd dirs gets masked anyway
            gold_f, gold_b = gold_f[mask_f], gold_b[mask_b]
            loss = self.get_loss(f_logits, b_logits, gold_f, gold_b)
            if loss:
                loss.backward()
            nn.utils.clip_grad_norm_(self.lm.parameters(), 5.0)
            self.optimizer.step()
            self.scheduler.step()

    @torch.no_grad()
    def evaluate(self, loader, punct=True):
        self.lm.eval()

        loss, metric = 0, Metric()

        for words, _, chars, _, _ in loader:
            mask_f = words.ne(self.vocab.pad_index)
            mask_b = words.ne(self.vocab.pad_index)
            # ignore the last token of each sentence in forward dir
            mask_f[:, -1] = 0
            # ignore the first token of each sentence in backward dir
            mask_b[:, 0] = 0

            f_logits, b_logits = self.lm(words, chars)

            f_logits, b_logits = f_logits[mask_f], b_logits[mask_b]

            # this is a silly way to offset the targets
            # the first and last token in the bwd and fwd dirs get masked anyway
            gold_f = torch.cat([words[:, 1:], words[:, 0:1]], dim=1)
            gold_b = torch.cat([words[:,-1:], words[:, :-1]], dim=1)
            gold_f, gold_b = gold_f[mask_f], gold_b[mask_b]
            ###################################################################
            #### pass lens to decode if forcing well-formed tree during train #
            ### lens = mask.sum(dim=1).tolist()                               #
            ### pred_arcs, pred_rels = self.decode(s_arc, s_rel, lens)        #
            ###################################################################
            current_loss = self.get_loss(f_logits, b_logits, gold_f, gold_b)
            if current_loss:
                loss += current_loss

        loss /= len(loader)

        return loss


    def get_loss(self, f_logits, b_logits, gold_f, gold_b):
        
        try:
            f_loss = self.criterion(f_logits, gold_f)
            b_loss = self.criterion(b_logits, gold_b)
        except:
            return False
        loss = f_loss + b_loss

        return loss

    def calculate_normalized_entropy(self, probabilities):
        normalized_entropy = 0.0
        n = len(probabilities)
        for p in probabilities:
            normalized_entropy -= (p * log(p)) / log(n)
        return normalized_entropy


        return pred_arcs, pred_rels

