def seq_lab_encoder(tree):
    labels = encode(tree)
    
    enc = []
    for inode, node in enumerate(tree):
        enc.append(node + [labels[inode]])
    return enc

def seq_lab_decoder(flat_tree):
    heads, rels = [], []
    for inode, node in enumerate(flat_tree):
        heads.append(inode)
        rels.append("nmod")
    return heads, rels

def seq_lab_decoder_corpus(pred_enc, pred_rels):
    all_p_heads, all_p_rels = [], []
    for flat_tree, pred_rel in zip(pred_enc, pred_rels):
        p_head, p_rels = decode(flat_tree, pred_rel)
        if p_head:
            all_p_heads.append(p_head)
            all_p_rels.append(p_rels)
    return all_p_heads, all_p_rels


def encode(rootless_tree):
    split = ""#"@"
    tree = ["ROOT"] + rootless_tree
    labels = ["_"] * len(tree)
    for inode, node in enumerate(tree):
        if node == "ROOT":
            continue

        rel = ""#node[7]
        head = int(node[6])
        if len(tree) - 1 == 1:
            labels[inode] = str("." + split + rel)
            break

        elif inode + 1 < len(tree):
            next_node = tree[inode + 1]
            brackets = ""
            head_of_next_node = int(next_node[6])
            rel_of_next_node = ""#next_node[7]
            if inode == 1 and rel == "root":
                label = str("." + split + rel)
                labels[inode] = label
                
            # searching for symbols "<" and "/"
            # check if w(i) has an incoming arc coming from the right
            if head > inode:
                if inode == 1:
                    label = str("." + split + rel)
                    brackets += "<"
                    labels[inode] = label
                else:
                    brackets += "<"

            # check if w(i) has k outgoing arcs towards the right.

            for i, check_words in enumerate(tree[inode+1:]):
                head_of_words = int(check_words[6])
                if head_of_words == inode:
                    brackets += "R"

            # searching for symbols "\" and ">"

            for i, check_words in enumerate(tree[1:inode+1]):
                head_of_words = int(check_words[6])
                if head_of_words == inode + 1:
                    brackets += 'L'

            # check if w(i+1) has an incoming arc coming from the left ">"
            if head_of_next_node < inode + 1 and not head_of_next_node == 0:
                brackets += ">"

                # print(next_node[1] + "> " + next_node[5])
                # check if w(i+1) has k outgoing arcs towards the left "\"
            if brackets != "":
                label = str(brackets + split + rel_of_next_node)
                labels[inode+1] = label
            else:
                label = str("." + split + rel_of_next_node)
                labels[inode+1] = label

    return labels[1:]



def decode(encoded_sentence, rels):
    if len(encoded_sentence) == 0:
        return None, None
    brackets = ["ROOT"] + encoded_sentence
    decoded_rels = ["_"] + rels
    #for label in encoded_sentence:
        #items = label.split("@")
        #brackets.append(label)#items[0])
        #decoded_rels.append(items[1])
    decoded_heads = [0] + [-1 for x in encoded_sentence]
    stack1 = []
    stack2 = []
    arcs, heads = [], []
    encoded_sentence = ["ROOT"] + encoded_sentence
    #root_node = -1
    for inode, enc_lab in enumerate(encoded_sentence):
        if enc_lab == "ROOT":
            continue
        if not len(encoded_sentence) - 1 == 1:
            if inode + 1 < len(encoded_sentence):
                proceeding_bracket = brackets[inode+1]
                char = list(proceeding_bracket)
                #if decoded_rels[inode] == "root":
                #    if root_node == -1:
                #        decoded_heads[inode] = 0
                #        root_node = inode
                if proceeding_bracket == "<L":
                    decoded_heads[inode] = int(inode) + 1
                elif proceeding_bracket == "R>":
                    decoded_heads[inode+1] = int(inode)
                else:
                    for x in char:
                        if x == "<" or x == "L":
                            x += "_" + repr(inode + 1)
                            stack1.append(x)
                            if len(stack1) > 1:
                                for w in range(0, len(stack1), 1):
                                    if w + 1 < len(stack1):
                                        s1 = stack1[w].rsplit("_", 1)
                                        s2 = stack1[w + 1].rsplit("_", 1)
                                        symbol = s1[0] + s2[0]
                                        if symbol == "<L":
                                            pos = int(s1[1]) - 1
                                            head = int(s2[1])
                                            decoded_heads[pos] = head
                                            w1 = stack1[w]
                                            w2 = stack1[w + 1]
                                            stack1.remove(w1)
                                            stack1.remove(w2)
                        if x == ">" or x == "R":
                            x += "_" + repr(inode + 1)
                            stack2.append(x)
                            if len(stack2) > 1:
                                for w in range(0, len(stack2), 1):
                                    if w + 1 < len(stack2):
                                        s1 = stack2[w].rsplit("_", 1)
                                        s2 = stack2[w + 1].rsplit("_", 1)
                                        symbol = s1[0] + s2[0]
                                        if symbol == "R>":
                                            pos = int(s2[1])
                                            head = int(s1[1]) - 1
                                            decoded_heads[pos] = head
                                            w1 = stack2[w]
                                            w2 = stack2[w + 1]
                                            stack2.remove(w1)
                                            stack2.remove(w2)

    decoded_heads, decoded_rels = post_process(decoded_heads, decoded_rels)
    return decoded_heads[1:], decoded_rels[1:]


def post_process(decoded_heads, decoded_rels):
    try:
        root_node = decoded_heads[1:].index(0)
    except:
        root_node = 1
    decoded_rels[root_node] = "root"
    decoded_heads[root_node] = 0
    for node, dh in enumerate(decoded_heads):
        if dh == "ROOT":
            continue
        if dh == -1:
            decoded_heads[node] = root_node
        if dh == 0 and node != root_node:
            decoded_heads[node] = root_node
            if decoded_rels[node] == "root":
                decoded_rels[node] = "x"
    decoded_heads = remove_cycles(decoded_heads, root_node)
    return decoded_heads, decoded_rels


def remove_cycles(pred_heads, root_node):
    arcs = [(head, node) for node, head in enumerate(pred_heads)][1:]
    nodes = [node for node, head in enumerate(pred_heads)][1:]
    heads = pred_heads
    leaf = True        
    while leaf == True:
        leaf = False
        for node in nodes:
            if node not in heads:
                arcs = remove_arcs(arcs, node)
                heads = [arc[0] for arc in arcs]
                nodes.remove(node)
                leaf = True
                break
    if len(arcs) > 0:
        for arc in arcs:
            node = arc[1]
            pred_heads[node] = root_node
    return pred_heads

def remove_arcs(arcs, node):
    new_arcs = [arc for arc in arcs if arc[1] != node]
    return new_arcs
