# -*- coding: utf-8 -*-
import energyusage
import argparse
import os, csv
from parser.cmds import Evaluate, Predict, Train, Pretrain, Forge, Flip, Increment, Score
import torch, random, numpy

from config import Config

def get_energy(cmd, config):
    cmd(config)

if __name__ == '__main__':

    seed_num = 17
    random.seed(seed_num)
    torch.manual_seed(seed_num)
    numpy.random.seed(seed_num)
    
    parser = argparse.ArgumentParser(
        description='Create the Biaffine Parser model.'
    )
    subparsers = parser.add_subparsers(title='Commands', dest='mode')
    subcommands = {
        'evaluate': Evaluate(),
        'predict': Predict(),
        'train': Train(),
        'pretrain': Pretrain(),
        'flip': Flip(),
        'increment': Increment(),
        'forge': Forge(),
        'score': Score()
    }
    for name, subcommand in subcommands.items():
        subparser = subcommand.add_subparser(name, subparsers)
        subparser.add_argument('--conf', '-c', default='config.ini',
                               help='path to config file')
        subparser.add_argument('--model', '-m', default='exp/ptb/model.char',
                               help='path to model file')
        subparser.add_argument('--vocab', '-v', default='exp/ptb/vocab.char',
                               help='path to vocab file')
        subparser.add_argument('--device', '-d', default='-1',
                               help='ID of GPU to use')
        subparser.add_argument('--seed', '-s', default=1, type=int,
                               help='seed for generating random numbers')
        subparser.add_argument('--threads', '-t', default=4, type=int,
                               help='max num of threads')
        subparser.add_argument('--log_energy',  default=None,
                               help='path to save energy usage')
    args = parser.parse_args()

    print(f"Set the max num of threads to {args.threads}")
    print(f"Set the seed for generating random numbers to {args.seed}")
    print(f"Set the device with ID {args.device} visible")
    torch.set_num_threads(args.threads)
    torch.manual_seed(args.seed)
    os.environ['CUDA_VISIBLE_DEVICES'] = args.device

    print(f"Override the default configs with parsed arguments")
    config = Config(args.conf)
    config.update(vars(args))
    print(config)

    print(f"Run the subcommand in mode {args.mode}")
    cmd = subcommands[args.mode]
    if config.log_energy is not None:
        time, energy, x = energyusage.evaluate(get_energy, cmd, config, printToScreen=False, energyOutput=True)
        if os.path.exists(config.log_path):
            with open(config.log_energy, mode="a") as output:
                output_writer = csv.writer(output, delimiter=',')
                output_writer.writerow(["seqlab-"+config.model.split("/")[-1],
                                        str(energy*3.6e+3)])
        else:
            with open(config.log_energy, mode="w") as output:
                output_writer = csv.writer(output, delimiter=',')
                output_writer.writerow(["Model", "Energy (kJ)"])
                output_writer.writerow(["seqlab-"+config.model.split("/")[-1],
                                        str(energy*3.6e+3)])
    else:
        cmd(config)
